import * as i0 from '@angular/core';
import { InjectionToken, Injectable, Optional, Inject, EventEmitter, Directive, Input, Output, HostListener, HostBinding, ContentChild, NgModule } from '@angular/core';
import { __awaiter } from 'tslib';
import { map, debounceTime, takeWhile } from 'rxjs/operators';
import { Subject, fromEvent } from 'rxjs';

function createXhr() {
    return new XMLHttpRequest();
}
function releaseXhr(xhr) {
    xhr = null;
}
class UploadxAjax {
    constructor(buildXhr) {
        this.buildXhr = buildXhr;
        this.request = ({ method = 'GET', data = null, headers = {}, url, responseType, canceler, onUploadProgress, timeout = 0, withCredentials = false, validateStatus = status => status < 400 && status >= 200 }) => {
            const xhr = this.buildXhr();
            canceler && (canceler.onCancel = () => xhr && xhr.readyState !== xhr.DONE && xhr.abort());
            return new Promise((resolve, reject) => {
                xhr.open(method, url, true);
                xhr.timeout = timeout;
                withCredentials && (xhr.withCredentials = true);
                responseType && (xhr.responseType = responseType);
                Object.keys(headers).forEach(key => xhr.setRequestHeader(key, String(headers[key])));
                xhr.upload.onprogress = onUploadProgress || null;
                xhr.onerror =
                    xhr.ontimeout =
                        xhr.onabort =
                            evt => {
                                releaseXhr(xhr);
                                return reject({ error: evt.type, url, method });
                            };
                xhr.onload = () => {
                    const response = {
                        data: this.getResponseBody(xhr),
                        status: xhr.status,
                        headers: this.getResponseHeaders(xhr)
                    };
                    releaseXhr(xhr);
                    return validateStatus(response.status) ? resolve(response) : reject(response);
                };
                xhr.send(data);
            });
        };
    }
    getResponseHeaders(xhr) {
        const rows = xhr.getAllResponseHeaders().split(/[\r\n]+/);
        return rows.reduce((headers, current) => {
            const [name, value] = current.split(': ');
            name && (headers[name.toLowerCase()] = value);
            return headers;
        }, {});
    }
    getResponseBody(xhr) {
        if (xhr.responseType === 'document') {
            return 'response' in xhr ? xhr.response : xhr.responseXML;
        }
        let body = 'response' in xhr ? xhr.response : xhr.responseText;
        if (xhr.responseType === 'json') {
            if (body && typeof body === 'string') {
                try {
                    body = JSON.parse(body);
                }
                catch (_a) { }
            }
        }
        return body;
    }
}
const UPLOADX_AJAX = new InjectionToken('uploadx.ajax', {
    factory: () => new UploadxAjax(createXhr),
    providedIn: 'root'
});

class Canceler {
    constructor() {
        this.onCancel = () => { };
    }
    cancel() {
        this.onCancel();
        this.onCancel = () => { };
    }
}

const KiB = 1024;
/**
 * Adaptive chunk size
 */
class DynamicChunk {
    static scale(throughput) {
        const elapsedTime = DynamicChunk.size / throughput;
        if (elapsedTime < DynamicChunk.minChunkTime) {
            DynamicChunk.size = Math.min(DynamicChunk.maxSize, DynamicChunk.size * 2);
        }
        if (elapsedTime > DynamicChunk.maxChunkTime) {
            DynamicChunk.size = Math.max(DynamicChunk.minSize, DynamicChunk.size / 2);
        }
        return DynamicChunk.size;
    }
}
/** Maximum chunk size in bytes */
DynamicChunk.maxSize = Number.MAX_SAFE_INTEGER;
/** Minimum chunk size in bytes */
DynamicChunk.minSize = 256 * KiB;
/** Initial chunk size in bytes */
DynamicChunk.size = 4 * (256 * KiB);
DynamicChunk.minChunkTime = 8;
DynamicChunk.maxChunkTime = 24;

/* eslint-disable no-bitwise */
function safeMatch(base, re) {
    return (base.match(re) || [''])[0];
}
function resolveUrl(url, base) {
    if (url.indexOf('https://') * url.indexOf('http://') === 0)
        return url;
    if (url.indexOf('//') === 0)
        return safeMatch(base, /^(https?:)/) + url;
    if (url.indexOf('/') === 0)
        return safeMatch(base, /^(?:https?:)?(?:\/\/)?([^\/?]+)/) + url;
    return safeMatch(base, /^(?:https?:)?(?:\/\/)?([^\/?]+)?(.*\/)/) + url;
}
function unfunc(value, ref) {
    return value instanceof Function ? value(ref) : value;
}
const pick = (obj, props) => {
    const result = {};
    props.forEach(key => (result[key] = obj[key]));
    return result;
};
function isNumber(x) {
    return x === Number(x);
}
/**
 * 32-bit FNV-1a hash function
 */
function createHash(str) {
    let hash = 2166136261;
    const len = str.length;
    for (let i = 0; i < len; i++) {
        hash ^= str.charCodeAt(i);
        hash += (hash << 1) + (hash << 4) + (hash << 7) + (hash << 8) + (hash << 24);
    }
    return hash >>> 0;
}
const b64 = {
    encode: (str) => btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g, (match, p1) => String.fromCharCode(Number.parseInt(p1, 16)))),
    decode: (str) => decodeURIComponent(atob(str)
        .split('')
        .map(c => '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2))
        .join('')),
    serialize: (obj) => Object.keys(obj)
        .map(key => [key, b64.encode(String(obj[key]))].filter(Boolean).join(' '))
        .toString(),
    parse: (encoded) => {
        const kvPairs = encoded.split(',').map(kv => kv.split(' '));
        const decoded = {};
        for (const [key, value] of kvPairs) {
            if (key)
                decoded[key] = value ? b64.decode(value) : '';
        }
        return decoded;
    }
};
function isBrowser() {
    return ![typeof window, typeof navigator].includes('undefined');
}
function isIOS() {
    return (isBrowser() &&
        (/iPad|iPhone|iPod/.test(navigator.platform) ||
            (navigator.maxTouchPoints > 2 && navigator.platform.includes('MacIntel'))));
}
function onLine() {
    return isBrowser() ? navigator.onLine : true;
}

class IdService {
    generateId(uploader) {
        const print = JSON.stringify(Object.assign(Object.assign({}, uploader.metadata), { type: uploader.constructor.name, endpoint: uploader.endpoint }));
        return createHash(uploader.name + uploader.size).toString(16) + createHash(print).toString(16);
    }
}
IdService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IdService, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
IdService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IdService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: IdService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

var ErrorType;
(function (ErrorType) {
    ErrorType[ErrorType["NotFound"] = 0] = "NotFound";
    ErrorType[ErrorType["Auth"] = 1] = "Auth";
    ErrorType[ErrorType["Retryable"] = 2] = "Retryable";
    ErrorType[ErrorType["Fatal"] = 3] = "Fatal";
})(ErrorType || (ErrorType = {}));
const defaultRetryConfig = {
    maxAttempts: 8,
    shouldRestartCodes: [404, 410],
    authErrorCodes: [401],
    shouldRetryCodes: [408, 423, 429, 460],
    shouldRetry(code) {
        return code < 400 || code >= 500 || this.shouldRetryCodes.includes(code);
    },
    minDelay: 500,
    maxDelay: 50000,
    onBusyDelay: 1000,
    timeout: 0
};
/**
 * Retryable ErrorHandler
 */
class RetryHandler {
    constructor(configOptions = {}) {
        this.attempts = 0;
        this.cancel = () => { };
        this.config = Object.assign({}, defaultRetryConfig, configOptions);
    }
    kind(code) {
        this.attempts++;
        if (this.attempts > this.config.maxAttempts) {
            return ErrorType.Fatal;
        }
        if (this.config.authErrorCodes.includes(code)) {
            return ErrorType.Auth;
        }
        if (this.config.shouldRestartCodes.includes(code)) {
            return ErrorType.NotFound;
        }
        if (this.config.shouldRetry(code, this.attempts)) {
            return ErrorType.Retryable;
        }
        return ErrorType.Fatal;
    }
    wait(time) {
        const ms = time || Math.min(Math.pow(2, (this.attempts - 1)) * this.config.minDelay, this.config.maxDelay);
        const jitter = Math.floor(Math.random() * this.config.minDelay * this.attempts);
        let id;
        return new Promise(resolve => {
            this.cancel = () => {
                clearTimeout(id);
                resolve();
            };
            id = setTimeout(this.cancel, ms + jitter);
        });
    }
    observe(value) {
        this.observedValue !== value && (this.attempts = 0);
        this.observedValue = value;
    }
}

const HOUR = 1000 * 60 * 60;
class Store {
    constructor(prefix = 'UPLOADX-v4.0-') {
        this.prefix = prefix;
        this.ttl = 24 * HOUR;
    }
    set(key, value) {
        this.ttl &&
            localStorage.setItem(this.prefix + key, JSON.stringify([value, Date.now() + this.ttl]));
    }
    get(key) {
        const item = localStorage.getItem(this.prefix + key);
        if (item) {
            const [value, expires] = JSON.parse(item);
            return value && expires ? value : null;
        }
        return null;
    }
    delete(key) {
        localStorage.removeItem(this.prefix + key);
    }
    clear(maxAgeHours = 0) {
        this.ttl = maxAgeHours * HOUR;
        const now = Date.now();
        this.keys().forEach(key => {
            const item = localStorage.getItem(key);
            if (item && maxAgeHours) {
                const [, expires] = JSON.parse(item);
                now > Number(expires) && localStorage.removeItem(key);
            }
            else {
                localStorage.removeItem(key);
            }
        });
    }
    keys() {
        return Object.keys(localStorage).filter(key => key.indexOf(this.prefix) === 0);
    }
}
const store = isLocalStorageAvailable() ? new Store() : new Map();
function isLocalStorageAvailable() {
    try {
        const key = 'LocalStorageTest';
        const value = 'value';
        localStorage.setItem(key, value);
        const getValue = localStorage.getItem(key);
        localStorage.removeItem(key);
        return getValue === value;
    }
    catch (_a) {
        return false;
    }
}

const actionToStatusMap = {
    pause: 'paused',
    upload: 'queue',
    cancel: 'cancelled'
};
/**
 * Uploader Base Class
 */
class Uploader {
    constructor(file, options, stateChange, ajax) {
        this.file = file;
        this.options = options;
        this.stateChange = stateChange;
        this.ajax = ajax;
        this.response = null;
        this.responseStatus = 0;
        this.responseHeaders = {};
        this.progress = 0;
        this.speed = 0;
        /** Custom headers */
        this.headers = {};
        /** Upload endpoint */
        this.endpoint = '/upload';
        this.canceler = new Canceler();
        this._eventsCount = 0;
        this._url = '';
        /**
         * Set auth token string
         */
        this.updateToken = () => {
            return unfunc(this.token || '', this.responseStatus);
        };
        this.cleanup = () => store.delete(this.uploadId);
        this.retry = new RetryHandler(options.retryConfig);
        this.name = file.name;
        this.size = file.size;
        this.metadata = {
            name: file.name,
            mimeType: file.type || 'application/octet-stream',
            size: file.size,
            lastModified: file.lastModified
        };
        options.maxChunkSize && (DynamicChunk.maxSize = options.maxChunkSize);
        this._prerequest = options.prerequest || (req => req);
        this._authorize = options.authorize || (req => req);
        this.configure(options);
    }
    get url() {
        return this._url || store.get(this.uploadId) || '';
    }
    set url(value) {
        this._url !== value && store.set(this.uploadId, value);
        this._url = value;
    }
    get status() {
        return this._status;
    }
    set status(s) {
        if (this._status === 'cancelled' || (this._status === 'complete' && s !== 'cancelled')) {
            return;
        }
        if (s !== this._status) {
            this.status === 'retry' && this.retry.cancel();
            this._status = s;
            s === 'paused' && this.abort();
            ['cancelled', 'complete', 'error'].includes(s) && this.cleanup();
            s === 'cancelled' ? this.cancel() : this.stateChange(this);
        }
    }
    /**
     * Configure uploader
     */
    configure({ metadata, headers, token, endpoint, action }) {
        endpoint && (this.endpoint = endpoint);
        token && (this.token = token);
        metadata && Object.assign(this.metadata, unfunc(metadata, this.file));
        headers && Object.assign(this.headers, unfunc(headers, this.file));
        action && (this.status = actionToStatusMap[action]);
    }
    /**
     * Starts uploading
     */
    upload() {
        return __awaiter(this, void 0, void 0, function* () {
            this._status = 'uploading';
            while (this.status === 'uploading' || this.status === 'retry') {
                this.status = 'uploading';
                try {
                    this._token || (this._token = yield this.updateToken());
                    this.url || (this.url = yield this.getFileUrl());
                    if (this.offset !== this.size) {
                        this.offset = isNumber(this.offset)
                            ? yield this.sendFileContent()
                            : yield this.getOffset();
                    }
                    this.retry.observe(this.offset);
                    if (this.offset === this.size) {
                        this.remaining = 0;
                        this.progress = 100;
                        this.status = 'complete';
                    }
                    else if (!this.offset) {
                        this.stateChange(this);
                        yield this.retry.wait(this.getRetryAfterFromBackend() || this.retry.config.onBusyDelay);
                    }
                }
                catch (e) {
                    e instanceof Error && console.error(e);
                    if (this.status !== 'uploading') {
                        return;
                    }
                    switch (this.retry.kind(this.responseStatus)) {
                        case ErrorType.Fatal:
                            this.status = 'error';
                            return;
                        case ErrorType.NotFound:
                            this.url = '';
                            break;
                        case ErrorType.Auth:
                            this._token = '';
                            break;
                        default:
                            if (this.responseStatus >= 400 || this.chunkSize > DynamicChunk.size) {
                                this.offset = undefined;
                            }
                            this.status = 'retry';
                            yield this.retry.wait(this.getRetryAfterFromBackend());
                    }
                }
            }
        });
    }
    /**
     * Performs http requests
     */
    request(requestOptions) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.responseStatus = 0;
            this.response = null;
            this.responseHeaders = {};
            let req = {
                body: requestOptions.body || null,
                canceler: this.canceler,
                headers: Object.assign(Object.assign({}, this.headers), requestOptions.headers),
                method: requestOptions.method || 'GET',
                url: requestOptions.url || this.url
            };
            req = yield this._authorize(req, this._token);
            const { body = null, headers, method, url = req.url } = (yield this._prerequest(req)) || req;
            const ajaxRequestConfig = {
                method,
                headers: Object.assign(Object.assign({}, req.headers), headers),
                url,
                data: body,
                responseType: (_a = this.options.responseType) !== null && _a !== void 0 ? _a : this.responseType,
                withCredentials: !!this.options.withCredentials,
                canceler: this.canceler,
                validateStatus: () => true,
                timeout: this.retry.config.timeout
            };
            //todo: more reliable detection
            if (isNumber(this.offset) && body && typeof body === 'object') {
                ajaxRequestConfig.onUploadProgress = this.onProgress();
            }
            const response = yield this.ajax.request(ajaxRequestConfig);
            this.response = response.data;
            this.responseHeaders = response.headers;
            this.responseStatus = response.status;
            if (response.status >= 400) {
                return Promise.reject();
            }
        });
    }
    abort() {
        this.offset = undefined;
        this.canceler.cancel();
    }
    cancel() {
        return __awaiter(this, void 0, void 0, function* () {
            this.abort();
            if (this.url) {
                yield this.request({ method: 'DELETE' }).catch(() => { });
            }
            this.stateChange(this);
        });
    }
    /**
     * Gets the value from the response
     */
    getValueFromResponse(key) {
        return this.responseHeaders[key.toLowerCase()] || null;
    }
    getChunk() {
        this.chunkSize =
            this.options.chunkSize === 0 ? this.size : this.options.chunkSize || DynamicChunk.size;
        const start = this.offset || 0;
        const end = Math.min(start + this.chunkSize, this.size);
        const body = this.file.slice(this.offset, end);
        return { start, end, body };
    }
    getRetryAfterFromBackend() {
        return Number(this.getValueFromResponse('retry-after')) * 1000;
    }
    onProgress() {
        let throttle;
        const startTime = Date.now();
        return ({ loaded }) => {
            const current = loaded / ((Date.now() - startTime) / 1000);
            this.speed = ~~((this.speed * this._eventsCount + current) / ++this._eventsCount);
            DynamicChunk.scale(this.speed);
            if (!throttle) {
                throttle = setTimeout(() => (throttle = undefined), 500);
                const uploaded = (this.offset || 0) + loaded;
                this.progress = +((uploaded / this.size) * 100).toFixed(2);
                this.remaining = ~~((this.size - uploaded) / this.speed);
                this.stateChange(this);
            }
        };
    }
}

/**
 * Implements XHR/CORS Resumable Upload
 *
 * {@link https://github.com/kukhariev/node-uploadx/blob/master/proto.md|Github}
 * @see {@link https://developers.google.com/drive/api/v3/manage-uploads#resumable|Google Drive API documentation}
 */
class UploaderX extends Uploader {
    constructor() {
        super(...arguments);
        this.responseType = 'json';
    }
    getFileUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const body = JSON.stringify(this.metadata);
            const headers = {
                'Content-Type': 'application/json; charset=utf-8',
                'X-Upload-Content-Length': this.size,
                'X-Upload-Content-Type': this.file.type || 'application/octet-stream'
            };
            yield this.request({ method: 'POST', body, url: this.endpoint, headers });
            this.offset = this.getOffsetFromResponse() || (this.responseStatus === 201 ? 0 : undefined);
            const location = this.getValueFromResponse('location');
            if (!location) {
                throw new Error('Invalid or missing Location header');
            }
            return resolveUrl(location, this.endpoint);
        });
    }
    sendFileContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const { body, start, end } = this.getChunk();
            const headers = {
                'Content-Type': 'application/octet-stream',
                'Content-Range': `bytes ${start}-${end - 1}/${this.size}`
            };
            yield this.request({ method: 'PUT', body, headers });
            return this.responseStatus > 201 ? this.getOffsetFromResponse() : end;
        });
    }
    getOffset() {
        return __awaiter(this, void 0, void 0, function* () {
            const headers = {
                'Content-Type': 'application/octet-stream',
                'Content-Range': `bytes */${this.size}`
            };
            yield this.request({ method: 'PUT', headers });
            return this.responseStatus > 201 ? this.getOffsetFromResponse() : this.size;
        });
    }
    getOffsetFromResponse() {
        const range = this.getValueFromResponse('Range');
        return range ? getRangeEnd(range) + 1 : undefined;
    }
}
function getRangeEnd(range = '') {
    const end = parseInt(range.split(/-/)[1], 10);
    return end >= 0 ? end : -1;
}

const defaultOptions = {
    endpoint: '/upload',
    autoUpload: true,
    concurrency: 2,
    uploaderClass: UploaderX,
    authorize: (req, token) => {
        token && (req.headers['Authorization'] = `Bearer ${token}`);
        return req;
    },
    storeIncompleteHours: 24
};
const UPLOADX_FACTORY_OPTIONS = new InjectionToken('uploadx.factory.options', { providedIn: 'root', factory: () => defaultOptions });
const UPLOADX_OPTIONS = new InjectionToken('uploadx.options');
const iOSPatch = (options) => {
    if (options.chunkSize || options.maxChunkSize) {
        console.warn('Detected iOS device, chunk uploads are problematic on this device.');
        return;
    }
    console.warn('Detected iOS device, chunk uploading and retries on errors are disabled.');
    options.chunkSize = 0;
    options.retryConfig = { shouldRetry: () => false };
};

/**
 * Implements tus resumable upload protocol
 *
 * {@link https://github.com/tus/tus-resumable-upload-protocol/blob/master/protocol.md|Github}
 */
class Tus extends Uploader {
    constructor() {
        super(...arguments);
        this.headers = { 'Tus-Resumable': '1.0.0' };
    }
    getFileUrl() {
        return __awaiter(this, void 0, void 0, function* () {
            const encodedMetaData = b64.serialize(this.metadata);
            const headers = {
                'Upload-Length': this.size,
                'Upload-Metadata': encodedMetaData
            };
            yield this.request({ method: 'POST', url: this.endpoint, headers });
            this.offset = this.getOffsetFromResponse() || (this.responseStatus === 201 ? 0 : undefined);
            const location = this.getValueFromResponse('location');
            if (!location) {
                throw new Error('Invalid or missing Location header');
            }
            return resolveUrl(location, this.endpoint);
        });
    }
    sendFileContent() {
        return __awaiter(this, void 0, void 0, function* () {
            const { body, start, end } = this.getChunk();
            const headers = {
                'Content-Type': 'application/offset+octet-stream',
                'Upload-Offset': start
            };
            yield this.request({ method: 'PATCH', body, headers });
            return this.getOffsetFromResponse() || end;
        });
    }
    getOffset() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.request({ method: 'HEAD' });
            return this.getOffsetFromResponse();
        });
    }
    getOffsetFromResponse() {
        const offset = this.getValueFromResponse('Upload-Offset');
        return offset ? parseInt(offset, 10) : undefined;
    }
}

const UPLOAD_STATE_KEYS = [
    'file',
    'name',
    'progress',
    'remaining',
    'response',
    'responseHeaders',
    'responseStatus',
    'size',
    'speed',
    'status',
    'uploadId',
    'url'
];
const DUE_TIME = 5;
class UploadxService {
    constructor(options, defaults, ajax, ngZone, idService) {
        this.ajax = ajax;
        this.ngZone = ngZone;
        this.idService = idService;
        /** Upload Queue */
        this.queue = [];
        this.eventsStream = new Subject();
        this.subs = [];
        this.stateChange = (uploader) => {
            this.ngZone.run(() => this.eventsStream.next(pick(uploader, UPLOAD_STATE_KEYS)));
            if (uploader.status !== 'uploading' && uploader.status !== 'added') {
                this.ngZone.runOutsideAngular(() => setTimeout(() => this.processQueue()));
            }
        };
        this.options = Object.assign({}, defaults, options);
        if (isBrowser()) {
            this.subs.push(fromEvent(window, 'online').subscribe(() => this.control({ action: 'upload' })), fromEvent(window, 'offline').subscribe(() => this.control({ action: 'pause' })));
        }
    }
    /** Upload status events */
    get events() {
        return this.eventsStream.asObservable();
    }
    /**
     * Initializes service
     * @param options global module options
     * @returns Observable that emits a new value on progress or status changes
     */
    init(options = {}) {
        Object.assign(this.options, options);
        return this.events;
    }
    /**
     * Initializes service
     * @param options global module options
     * @returns Observable that emits the current array of uploaders
     */
    connect(options) {
        return this.init(options).pipe(map(() => this.queue), debounceTime(DUE_TIME));
    }
    /**
     * Terminates all uploads and clears the queue
     */
    disconnect() {
        this.queue.forEach(uploader => (uploader.status = 'paused'));
        this.queue = [];
    }
    /**
     * Returns current uploads state
     * @example
     * // restore background uploads
     * this.uploads = this.uploadService.state();
     */
    state() {
        return this.queue.map(uploader => pick(uploader, UPLOAD_STATE_KEYS));
    }
    ngOnDestroy() {
        this.disconnect();
        this.subs.forEach(sub => sub.unsubscribe());
    }
    /**
     * Creates uploaders for files and adds them to the upload queue
     */
    handleFiles(files, options = {}) {
        isIOS() && iOSPatch(options);
        const instanceOptions = Object.assign(Object.assign({}, this.options), options);
        store.clear(instanceOptions.storeIncompleteHours);
        this.options.concurrency = instanceOptions.concurrency;
        ('name' in files ? [files] : Array.from(files)).forEach(file => this.addUploaderInstance(file, instanceOptions));
    }
    /**
     * Upload control
     * @example
     * // pause all
     * this.uploadService.control({ action: 'pause' });
     * // pause upload with uploadId
     * this.uploadService.control({ action: 'pause', uploadId});
     * // set token
     * this.uploadService.control({ token: `TOKEN` });
     */
    control(evt) {
        const target = evt.uploadId
            ? this.queue.filter(({ uploadId }) => uploadId === evt.uploadId)
            : this.queue;
        target.forEach(uploader => uploader.configure(evt));
    }
    /**
     * Number of active uploads
     */
    get activeUploadsCount() {
        return this.queue.filter(({ status }) => status === 'uploading' || status === 'retry').length;
    }
    /**
     * Performs http requests
     */
    request(config) {
        return __awaiter(this, void 0, void 0, function* () {
            config.data || (config.data = config.body);
            return this.ajax.request(config);
        });
    }
    addUploaderInstance(file, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uploader = new options.uploaderClass(file, options, this.stateChange, this.ajax);
            uploader.uploadId = yield this.idService.generateId(uploader);
            this.queue.push(uploader);
            uploader.status = 'added';
            if (options.autoUpload && onLine()) {
                uploader.status = 'queue';
            }
        });
    }
    processQueue() {
        this.queue = this.queue.filter(({ status }) => status !== 'cancelled');
        this.queue
            .filter(({ status }) => status === 'queue')
            .slice(0, Math.max(this.options.concurrency - this.activeUploadsCount, 0))
            .forEach(uploader => uploader.upload());
    }
}
UploadxService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxService, deps: [{ token: UPLOADX_OPTIONS, optional: true }, { token: UPLOADX_FACTORY_OPTIONS }, { token: UPLOADX_AJAX }, { token: i0.NgZone }, { token: IdService }], target: i0.ɵɵFactoryTarget.Injectable });
UploadxService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxService, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [UPLOADX_OPTIONS]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [UPLOADX_FACTORY_OPTIONS]
                }] }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [UPLOADX_AJAX]
                }] }, { type: i0.NgZone }, { type: IdService }]; } });

class UploadxDirective {
    constructor(elementRef, renderer, uploadService) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.uploadService = uploadService;
        this.options = {};
        this.state = new EventEmitter();
    }
    set uploadx(value) {
        if (value) {
            this.options = value;
        }
    }
    set control(value) {
        if (value) {
            this.uploadService.control(value);
        }
    }
    ngOnInit() {
        const { multiple, allowedTypes } = Object.assign(Object.assign({}, this.uploadService.options), this.options);
        multiple !== false && this.renderer.setAttribute(this.elementRef.nativeElement, 'multiple', '');
        allowedTypes &&
            this.renderer.setAttribute(this.elementRef.nativeElement, 'accept', allowedTypes);
        this.uploadService.events
            .pipe(takeWhile(() => this.state.observers.length > 0))
            .subscribe(this.state);
    }
    fileListener(files) {
        if (files && files.item(0)) {
            this.uploadService.handleFiles(files, this.options);
        }
    }
}
UploadxDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxDirective, deps: [{ token: i0.ElementRef }, { token: i0.Renderer2 }, { token: UploadxService }], target: i0.ɵɵFactoryTarget.Directive });
UploadxDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.16", type: UploadxDirective, selector: "[uploadx]", inputs: { uploadx: "uploadx", options: "options", control: "control" }, outputs: { state: "state" }, host: { listeners: { "change": "fileListener($event.target.files)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[uploadx]' }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.Renderer2 }, { type: UploadxService }]; }, propDecorators: { uploadx: [{
                type: Input
            }], options: [{
                type: Input
            }], control: [{
                type: Input
            }], state: [{
                type: Output
            }], fileListener: [{
                type: HostListener,
                args: ['change', ['$event.target.files']]
            }] } });

class UploadxDropDirective {
    constructor(uploadService) {
        this.uploadService = uploadService;
        this.active = false;
    }
    dropHandler(event) {
        this._stopEvents(event);
        this.active = false;
        if (event.dataTransfer && event.dataTransfer.files.item(0)) {
            this.fileInput
                ? this.fileInput.fileListener(event.dataTransfer.files)
                : this.uploadService.handleFiles(event.dataTransfer.files);
        }
    }
    onDragOver(event) {
        this._stopEvents(event);
        if (event.dataTransfer && event.dataTransfer.files.item(0)) {
            event.dataTransfer.dropEffect = 'copy';
            this.active = true;
        }
    }
    onDragLeave(event) {
        this._stopEvents(event);
        this.active = false;
    }
    _stopEvents(event) {
        event.stopPropagation();
        event.preventDefault();
    }
}
UploadxDropDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxDropDirective, deps: [{ token: UploadxService }], target: i0.ɵɵFactoryTarget.Directive });
UploadxDropDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.16", type: UploadxDropDirective, selector: "[uploadxDrop]", host: { listeners: { "drop": "dropHandler($event)", "dragover": "onDragOver($event)", "dragleave": "onDragLeave($event)" }, properties: { "class.uploadx-drop-active": "this.active" } }, queries: [{ propertyName: "fileInput", first: true, predicate: UploadxDirective, descendants: true }], ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxDropDirective, decorators: [{
            type: Directive,
            args: [{ selector: '[uploadxDrop]' }]
        }], ctorParameters: function () { return [{ type: UploadxService }]; }, propDecorators: { active: [{
                type: HostBinding,
                args: ['class.uploadx-drop-active']
            }], fileInput: [{
                type: ContentChild,
                args: [UploadxDirective, { static: false }]
            }], dropHandler: [{
                type: HostListener,
                args: ['drop', ['$event']]
            }], onDragOver: [{
                type: HostListener,
                args: ['dragover', ['$event']]
            }], onDragLeave: [{
                type: HostListener,
                args: ['dragleave', ['$event']]
            }] } });

class UploadxModule {
    static withConfig(options) {
        return {
            ngModule: UploadxModule,
            providers: [{ provide: UPLOADX_OPTIONS, useValue: options }]
        };
    }
}
UploadxModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
UploadxModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxModule, declarations: [UploadxDirective, UploadxDropDirective], exports: [UploadxDirective, UploadxDropDirective] });
UploadxModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.16", ngImport: i0, type: UploadxModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [UploadxDirective, UploadxDropDirective],
                    exports: [UploadxDirective, UploadxDropDirective]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { Canceler, DynamicChunk, ErrorType, IdService, RetryHandler, Store, Tus, UPLOADX_AJAX, UPLOADX_FACTORY_OPTIONS, UPLOADX_OPTIONS, UPLOAD_STATE_KEYS, Uploader, UploaderX, UploadxAjax, UploadxDirective, UploadxDropDirective, UploadxModule, UploadxService, b64, createHash, getRangeEnd, iOSPatch, isBrowser, isIOS, isLocalStorageAvailable, isNumber, onLine, pick, resolveUrl, store, unfunc };
//# sourceMappingURL=ngx-uploadx.js.map
